/*
 * Copyright (c) 1998-2005 Jitterbit, Inc.
 *
 */
package org.jitterbit.util.collections;


/**
 * Class that implements a wrapper around an <tt>Object</tt> and uses
 * object identity in its implementation of <tt>equals()</tt> and 
 * <tt>hashCode</tt>. This is useful in situations where one wants to 
 * guarantee that the <tt>contains()</tt> method of the <tt>Collection</tt>
 * interface will find an object, regardless of how the <tt>equals</tt>
 * and <tt>hashCode</tt> of the object itself has been implemented.
 *  
 * @author Torgil
 *
 */
public class IdentityWrapper<T> {

    private T  m_object;
    
    /**
     * Creates a IdentityWrapper for the specified <tt>Object</tt>.
     * @param object
     */
    public IdentityWrapper(T object) {
        m_object = object;
    }
    
    /**
     * Retrieves the object that is wrapped in this instance.
     */
    public T getObject() {
        return m_object;
    }

    /**
     * 
     * @return  <tt>true</tt> iff <tt>obj</tt> is an <tt>IdentityWrapper</tt>,
     *          and <tt>obj.getObject() == this.getObject()</tt>.
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
        return (obj instanceof IdentityWrapper) &&
            ((IdentityWrapper)obj).m_object == this.m_object;
    }

    /**
     * @return  the result of <tt>System.identityHashCode</tt>, passing the
     *          wrapped object as the argument.
     * @see System#identityHashCode(java.lang.Object)          
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        return System.identityHashCode(m_object);
    }

    /**
     * @return <tt>toString()</tt> called on the wrapped object, or 
     * <tt>null</tt> if the wrapped object is null.
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return (m_object == null) ? null : m_object.toString();
    }
    
    
    
}
