/*
 * Copyright (c) 1998-2006 Jitterbit, Inc.
 *
 */
package org.jitterbit.util.collections;

import java.util.Collection;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Set;

/**
 * 
 * @author Torgil
 *
 */
public class KongaCollectionUtils {

    private KongaCollectionUtils() {
        throw new UnsupportedOperationException();
    }
    
    /**
     * Creates and returns an empty iterator. The <tt>hasNext()</tt> method of
     * the returned iterator returns <tt>false</tt>. Calling <tt>next()</tt>
     * on the returned iterator results in a NoSuchElementException being
     * thrown; calling <tt>remove()</tt> results in a
     * UnsupportedOperationException being thrown.
     * 
     */
    public static <T> Iterator<T> getEmptyIterator() {
        return new Iterator<T>() {

            public boolean hasNext() {
                return false;
            }

            public T next() {
                throw new NoSuchElementException();
            }

            public void remove() {
                throw new UnsupportedOperationException();
            }
        };
    }

    /**
     * Creates and returns an empty Iterable.
     *  
     * @since 1.1
     */
    public static <T> Iterable<T> getEmptyIterable() {
        return new Iterable<T>() {

            public Iterator<T> iterator() {
                return getEmptyIterator();
            }
        };
    }
    
    /**
     * Retrieves a read-only iterator over the elements in <tt>collection</tt>.
     * Calling <tt>remove()</tt> on the returned iterator results in an
     * UnsupportedOperationException being thrown.
     * 
     * @param collection
     * @throws NullPointerException
     *             if <tt>collection</tt> is null.
     */
    public static <T> Iterator<T> getReadOnlyIterator(Collection<T> collection) {
        final Iterator<T> backing = collection.iterator();
        return new Iterator<T>() {

            public boolean hasNext() {
                return backing.hasNext();
            }

            public T next() {
                return backing.next();
            }

            public void remove() {
                throw new UnsupportedOperationException();
            }
        };
    }
    
    /**
     * Checks two Sets for equality. This method is null-safe: if one of the
     * Sets is null this method returns true only if the other Set is also null.
     * If both Sets are not-null, this method returns the result of
     * <tt>s1.equals(s2)</tt>.
     * 
     * @since 1.1
     */
    public static <T> boolean areEqual(Set<T> s1, Set<T> s2) {
        return (s1 == null) ? (s2 == null) : s1.equals(s2);
    }
}
