/*
 * Copyright (c) 1998-2005 Jitterbit, Inc.
 *
 */
package org.jitterbit.util.lang;

import java.io.ObjectStreamException;
import java.io.Serializable;
import java.util.*;

/**
 * Abstract class that should be used as a baseclass for enumerated types.
 * 
 * @author TorgilZ
 *
 */
public abstract class KongaEnum implements Serializable {

	// Each derived class will have its own set of class specific
	// information, such as what instances have been created. 
	// This information is stored in a [Class, ClassSpecificInfo]
	// map.
	private static final Map<Class, ClassSpecificInfo> s_classInfo = new HashMap<Class, ClassSpecificInfo>();

	// Sometimes, several derived classes shuold be treated as one class.
	// An example is org.jitterbit.edgeapp.data.DataType, where the derived
	// class (DataType) is abstract, and each individual instance in effect
	// constitute their own, anonymous class, derived from DataType. 
	// This [Class, Class] map translates from an actual class (e.g. an
	// anonymous subclass) to the baseclass that should be used to look
	// up the class specific information (DataType, in the example used here).
	private static final Map<Class, Class> s_classTranslator = new HashMap<Class, Class>();
	
	private int	m_iIndex;

	/**
	 * Creates a new <tt>KongaEnum</tt> instance. The supplied argument specifies
	 * the derived class that is being instantiated. Example of use:
	 * <pre>
	 * public class DerivedEnum extends KongaEnum {
	 * 
	 * 	public DerivedEnum() {
	 * 		super(DerivedEnum.class);
	 * 		...
	 * 	}
	 * 
	 * 	...
	 * 
	 * }
	 * </pre>
	 * @param derivedClass	A reference to a <tt>Class</tt> object representing
	 * the derived class.	
	 */
	protected KongaEnum(Class derivedClass) {
		s_classTranslator.put(getClass(), derivedClass);
		// Also map derivedClass to itself, for it may be used in
		// one of the static methods like getAllValues or fromIndex:
		s_classTranslator.put(derivedClass, derivedClass);
		ClassSpecificInfo info = getClassInfo();
		m_iIndex = info.getNextIndex();
		info.addInstance(this);
	}
	
	/**
	 * 
	 * @return The <code>ClassSpecificInfo</code> for this <code>KongaEnum</code>.
	 */
	protected final ClassSpecificInfo getClassInfo() {
		return getClassInfo(getClass());
	}
	
	/**
     * Returns the <code>ClassSpecificInfo</code> for the specified
     * <code>Class</code>.
     * 
     * @param cls0
     *            the <code>Class</code> for which we want the
     *            <code>ClassSpecificInfo</code>
     * @return The <code>ClassSpecificInfo</code> for the specified class.
     * @throws IllegalArgumentException
     *             if the submitted class is null or not an instance of
     *             <code>KongaEnum</code>.
     */
	private static ClassSpecificInfo getClassInfo(Class cls0) {
		if (cls0 == null) {
            throw new IllegalArgumentException("The specified class can't be null.");
        }
        if (!KongaEnum.class.isAssignableFrom(cls0)) {
            throw new IllegalArgumentException(
                            "The specified class must be an instanceof "
                                            + "KongaEnum. The submitted class was: "
                                            + cls0.toString());
        }
        Class cls = s_classTranslator.get(cls0);
        Object o = s_classInfo.get(cls);
        ClassSpecificInfo info = null;
        if (o instanceof ClassSpecificInfo) {
            info = (ClassSpecificInfo) o;
        }
        else {
            info = new ClassSpecificInfo();
            s_classInfo.put(cls, info);
        }
        return info;
	}
		
	// The information returned from this method is class specific, but it
	// can't be static since we need to know what class it is invoked on.
	/**
     * Returns all the instances of the same class as this. The same information
     * can be obtained by using the static method
     * <code>gettAllValues(java.lang.Class)</code>.
     * 
     * @return An array with all <code>KongaEnum</code> instances of the same
     *         class as this.
     * @see #getAllValues(java.lang.Class)
     */
	public final KongaEnum[] getAllValues() {
		ClassSpecificInfo info = getClassInfo();
		return info.getAllValues();
	}
	
	/**
     * Returns all the existing instances of the specified class.
     * 
     * @param cls
     *            the class for which we want a list of instances
     * @return an array that holds all the instances of the specified class
     * @throws IllegalArgumentException
     *             if the specified class is <code>null</code> or not an
     *             instance of <code>KongaEnum</code>.
     */
	public static final KongaEnum[] getAllValues(Class cls) {
		ClassSpecificInfo info = getClassInfo(cls);
		return info.getAllValues();
	}
	
	/**
     * Returns the <code>KongaEnum</code> instance of the same class as this,
     * with the specified index.
     * 
     * @param iIndex
     */
	public KongaEnum fromIndex(int iIndex) {
		ClassSpecificInfo info = getClassInfo();
		return info.getInstance(iIndex);
	}
	
	/**
     * Returns the <code>KongaEnum</code> instance of the specified class with
     * the specified index.
     * 
     * @param cls
     *            the class we are looking for
     * @param iIndex
     *            the index of the instance we are looking for
     * @return the <code>KongaEnum</code> instance of the specified class that
     *         has the specified index.
     * @throws IllegalArgumentException
     *             if the specified class is <code>null</code> or not an
     *             instance of <code>KongaEnum</code>.
     */
	public static KongaEnum fromIndex(Class cls, int iIndex) {
		ClassSpecificInfo info = getClassInfo(cls);
		return info.getInstance(iIndex);
	}

	/**
     * 
     * @return an int that is unique for this instance within this class.
     */
	public final int getIndex() {
		return m_iIndex;
	}
	
	// For serialization (to make sure there won't be any duplicates of the
	// KongaEnum instances after deserialization).
	protected final Object readResolve() throws ObjectStreamException {
		ClassSpecificInfo info = getClassInfo();
		return info.getInstance(m_iIndex);
	}
	
	
	
	/**
     * Class that keeps track of what instances have been created of a specific
     * class, and the current index to use for new instances of a specific
     * class.
     * 
     */
	private static class ClassSpecificInfo {
		
		private List<KongaEnum>	m_instances = new ArrayList<KongaEnum>();
		private int		m_nextIndex = 0;
		
		int getNextIndex() {
			return m_nextIndex++;
		}
		
		void addInstance(KongaEnum ewEnum) {
			m_instances.add(ewEnum);
		}
		
		KongaEnum getInstance(int iIndex) {
			return (iIndex >= 0 && iIndex < m_instances.size()) ? 
							m_instances.get(iIndex) : null;
		}
		
		KongaEnum[] getAllValues() {
			return m_instances.toArray(new KongaEnum[0]);
		}
		
		KongaEnum fromIndex(int iIndex) {
			return (iIndex >= 0 && iIndex < m_instances.size()) ? 
										m_instances.get(iIndex) : null;
		}
	}
	
	/**
	 * This implementation should be more efficient than the default...
	 */
    @Override
	public int hashCode() {
		// Multiplier to get more spread out values...
		return 10000*getIndex();
	}
}
