/*
 * Copyright (c) 1998-2006 Jitterbit, Inc.
 *
 */
package org.jitterbit.util.xml;

import java.util.HashSet;
import java.util.Set;

import org.w3c.dom.Node;

/**
 * Factory for creating <code>IQualifiedName</code> and
 * <code>IExpandedName</code> instances.
 * <p>
 * All implementations of this class are required to follow the contract
 * specified by {@link IQualifiedName} and {@link IExpandedName}: the local
 * part of a name created through the factory may not be null or empty.
 * Additional compliance to the XML standard regarding what constitutes a valid
 * name is optional. Two default factory implementations are provided:
 * <ul>
 * <li>{@link #conforming()}: Returns a factory that does not allow names to
 * be created that does not conform to the WC3 XML standard. The returned
 * factory will check that the created names does not contain illegal
 * characters, etc.</li>
 * <li>{@link #nonConforming()}: Returns a factory that does not do additional
 * checking of the created names, other than what is specified in the
 * <code>IQualifiedName</code> and <code>IExpandedName</code> contracts.</li>
 * </ul>
 * To illustrate:
 * <pre>
 * String s = "http://www.jitterbit.com/xml";
 * 
 * XmlNameFactory conforming = XmlNameFactory.conforming();
 * // The following method call results in an IllegalArgumentException, since
 * // the specified string cannot be parsed into an expanded name that conforms
 * // to the XML standard:
 * IExpandedName name = conforming.newExpandedName(s);
 * 
 * XmlNameFactory nonConforming = XmlNameFactory.nonConforming();
 * // The following method call returns an <code>IExpandedName</code>, the
 * // namespace URI part of which will be <code>"http"</code>, and the
 * // local part of which will be <code>"//www.jitterbit.com/xml"</code>.
 * name = nonConforming.newExpandedName(s);
 * </pre> 
 * 
 * @author Torgil
 * 
 * @since 1.2
 * 
 */
public abstract class XmlNameFactory {

    private static XmlNameFactory   s_conforming;
    private static XmlNameFactory   s_nonConforming;
    
    /**
     * Creates an <code>IExpandedName</code> with the specified namespace and
     * local parts.
     * 
     * @param namespace
     *            the namespace URI of the name. If this argument is
     *            <code>null</code> or a blank string, the resulting name has
     *            no namespace.
     * @param local
     *            the local part of the name.
     * @throws IllegalArgumentException
     *             if <code>local</code> is <code>null</code>, or if
     *             <code>local.trim() == 0</code>.
     */
    public abstract IExpandedName newExpandedName(String namespace, String local);

    /**
     * Returns an <code>IExpandedName</code> based on the specified string.
     * 
     * @throws IllegalArgumentException
     *             if <code>s</code> is
     *             <code>null<code>, if <code>s</code> is blank, or if <code>s</code> does not have a local part.
     */
    public abstract IExpandedName newExpandedName(String s);
    
    /**
     * Returns an <code>IExpandedName</code> representing the specified XML
     * DOM node.
     * 
     * @param node
     *            an <code>org.wc3.dom.Node</code> instance representing the
     *            node for which to construct an <code>IExpandedName</code>.
     * @throws NullPointerException
     *             if <code>node</code> is null.
     */
    public abstract IExpandedName newExpandedName(Node node);
    
    /**
     * Creates an <code>IQualifiedName</code> with the specified namespace
     * prefix and local part.
     * 
     * @param prefix
     *            the namespace prefix of the name. If this argument is
     *            <code>null</code> or a blank string, the resulting name is
     *            unprefixed.
     * @param local
     *            the local part of the name.
     * @throws IllegalArgumentException
     *             if <code>local</code> is <code>null</code>, or if
     *             <code>local.trim() == 0</code>.
     */
    public abstract IQualifiedName newQualifiedName(String prefix, String local);
    
    /**
     * Returns an <code>IQualifiedName</code> based on the specified string.
     * 
     * @throws IllegalArgumentException
     *             if <code>s</code> is
     *             <code>null<code>, if <code>s</code> is blank, or if <code>s</code> does not have a local part.
     */
    public abstract IQualifiedName newQualifiedName(String s);
    
    /**
     * Returns an <code>IQualifiedName</code> representing the specified XML
     * DOM node.
     * 
     * @param node
     *            an <code>org.wc3.dom.Node</code> instance representing the
     *            node for which to construct an <code>IQualifiedName</code>.
     * @throws NullPointerException
     *             if <code>node</code> is null.
     */
    public abstract IQualifiedName newQualifiedName(Node node);

    public synchronized static XmlNameFactory nonConforming() {
        if (s_nonConforming == null) {
            s_nonConforming = new FactoryImpl(false);
        }
        return s_nonConforming;
    }
    
    public synchronized static XmlNameFactory conforming() {
        if (s_conforming == null) {
            s_conforming = new FactoryImpl(true);
        }
        return s_conforming;
    }
    
    
    
    private static class FactoryImpl extends XmlNameFactory {

        private final boolean   m_conforming;
        
        public FactoryImpl(boolean conforming) {
            m_conforming = conforming;
        }
        
        @Override
        public IExpandedName newExpandedName(String namespace, String local) {
            return new Expanded(namespace, local, m_conforming);
        }

        @Override
        public IExpandedName newExpandedName(String s) {
            String[] parts = split(s);
            return new Expanded(parts[0], parts[1], m_conforming);
        }

        @Override
        public IExpandedName newExpandedName(Node node) {
            String uri = node.getNamespaceURI();
            String local = node.getLocalName();
            return new Expanded(uri, local, m_conforming);
        }

        @Override
        public IQualifiedName newQualifiedName(String prefix, String local) {
            return new Qualified(prefix, local, m_conforming);
        }

        @Override
        public IQualifiedName newQualifiedName(String s) {
            String[] parts = split(s);
            return new Qualified(parts[0], parts[1], m_conforming);
        }
        
        @Override
        public IQualifiedName newQualifiedName(Node node) {
            String prefix = node.getPrefix();
            String local = node.getLocalName();
            return new Qualified(prefix, local, m_conforming);
        }

        private String[] split(String s) {
            if ((s == null)) {
                throw new IllegalArgumentException("s cannot be null.");
            }
            String trimmed = s.trim();
            if (trimmed.length() == 0) {
                throw new IllegalArgumentException("s cannot be blank.");
            }
            int pos = trimmed.lastIndexOf(':');
            if (pos == (trimmed.length() - 1)) {
                throw new IllegalArgumentException("s does not contain a local part.");
            }
            String uri = (pos >= 0) ? trimmed.substring(0, pos) : "";
            String local = trimmed.substring(pos + 1);
            return new String[] { uri, local };
        }
    }
    
    
    private static final class Expanded extends Name implements IExpandedName {

        public Expanded(String namespace, String local, boolean validate) {
            super(namespace, local, validate);
        }
        
        public String getNamespace() {
            return getNamespacePart();
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == this) {
                return true;
            }
            if (obj instanceof IExpandedName) {
                IExpandedName that = (IExpandedName) obj;
                return getNamespace().equals(that.getNamespace()) && 
                    getLocalName().equals(that.getLocalName());
            }
            return false;
        }

        @Override
        public int hashCode() {
            int hash = 17;
            hash = 31 * hash + getNamespace().hashCode();
            hash = 31 * hash + getLocalName().hashCode();
            return hash;
        }
    }
    
    
    private static final class Qualified extends Name implements IQualifiedName {

        public Qualified(String prefix, String local, boolean validate) {
            super(prefix, local, validate);
        }
        
        public String getPrefix() {
            return getNamespacePart();
        }
    }
    
    
    private static abstract class Name {
        
        private final String    m_namespacePart;
        private final String    m_localName;
        
        protected Name(String namespacePart, String local, boolean validate) {
            if ((local == null) || (local.trim().length() == 0)) {
                throw new IllegalArgumentException("local cannot be null or empty.");
            }
            local = local.trim();
            if (validate) {
                validate(local);
            }
            if (namespacePart == null) {
                namespacePart = "";
            }
            m_namespacePart = namespacePart.trim();
            m_localName = local;
        }
        
        protected final String getNamespacePart() {
            return m_namespacePart;
        }
        
        public final String getLocalName() {
            return m_localName;
        }
        
        @Override
        public String toString() {
            if (m_namespacePart.length() == 0) {
                return m_localName;
            }
            return m_namespacePart + ":" + m_localName;
        }

        private static void validate(String localName) throws IllegalArgumentException {
            assert (localName != null) && (localName.trim().length() > 0);
            validateFirstCharacter(localName);
            validateRest(localName);
        }
        
        private static void validateFirstCharacter(String localName) {
            char c0 = localName.charAt(0);
            if (!isValidBeginningCharacter(c0)) {
                throw new IllegalArgumentException("The local name cannot start with a '" +
                                c0 + "'.");
            }
        }
        
        private static boolean isValidBeginningCharacter(char c) {
            return Character.isLetter(c) || (c == '_') || (c == ':');
        }

        private static void validateRest(String localName) {
            for (int n = 1, len = localName.length(); n < len; ++n) {
                char c = localName.charAt(n);
                if (!isValidNameCharacter(c)) {
                    throw new IllegalArgumentException("The local name cannot contain " +
                            "the character '" + c + "'");
                }
            }
        }
        
        private static boolean isValidNameCharacter(char c) {
            // TODO: Include "CombiningChars" and "Expanders", 
            // according to the WC3 document.
            return Character.isLetter(c) || Character.isDigit(c) ||
                (c == '.') || (c == '_') || (c == '-') || (c == ':');
        }
    }
    
    
    /**
     * Test code.
     * 
     */
    public static void main(String[] args) {
        XmlNameFactory f = nonConforming();
        String[] names = {
            "http://www.jitterbit.com/xml:Root",
            "http://www.jitterbit.com/xml:Child",
        };
        Set<IExpandedName> expNames = new HashSet<IExpandedName>();
        for (String name : names) {
            IExpandedName exp = f.newExpandedName(name);
            System.out.println(exp);
            expNames.add(exp);
            System.out.println(expNames.size());
        }
    }
}
