/*
 * Copyright (c) 1998-2006 Jitterbit, Inc.
 *
 */
package org.jitterbit.util.xml.sax;

import java.util.HashMap;
import java.util.Map;

import org.jitterbit.system.callback.IAccepter;

/**
 * Utility class that extracts the values of a number of "single elements", i.e
 * elements that has a single string value as their only content.
 * <p>
 * An instance of this class must be created before the XML parsing begins. When
 * the parser is done parsing, the values of the matching elements that were
 * encountered in the XML can be returned by calling the method
 * {@link #getValue(String)}. <strong>Note:</strong> This class does not do
 * any parsing itself. It merely acts as a callback when the specified parser
 * parses the XML.
 * 
 * @author Torgil
 * 
 * @since 1.2
 * 
 */
public final class SingleElementValueCollection {

    private final Map<String, String>   m_parsedValues;
    
    /**
     * Creates a single element value collection that will extract the values of
     * the specified elements.
     * 
     * @param parser
     *            the parser that will be doing the XML parser.
     * @param elements
     *            an array holding the names of the elements, the values of
     *            which will be collected in this collection.
     * @throws IllegalArgumentException
     *             if <code>parser</code> or <code>element</code> is null,
     *             or if <code>elements</code> contain null values.
     */
    public SingleElementValueCollection(DefaultKongaSaxParser parser,
                                        String[] elements) {
        if (parser == null) {
            throw new IllegalArgumentException("parser cannot be null.");
        }
        if (elements == null) {
            throw new IllegalArgumentException("elements cannot be null.");
        }
        m_parsedValues = new HashMap<String, String>();
        configureParser(parser, elements);
    }
    
    private void configureParser(DefaultKongaSaxParser parser, String[] elements) {
        for (final String e : elements) {
            if (e == null) {
                throw new IllegalArgumentException("elements may not contain null values.");
            }
            SingleElementValueHandler.register(parser, e, new IAccepter<String>() {

                public boolean accept(String t) {
                    m_parsedValues.put(e, t);
                    return true;
                }
            });
        }
    }

    /**
     * Returns the value of the specified element. This method should be called
     * once the parser is done parsing the XML content.
     * 
     * @param element
     *            the name of the element for which to return the value.
     * @return the value of the specified element, or <code>null</code> if the
     *         requested element was not encountered.
     */
    public String getValue(String element) {
        return m_parsedValues.get(element);
    }
    
    /**
     * Returns a copy of the current values.
     * 
     * @return a <code>Map</code>, containing the key-value pairs that have
     *         been parsed so far.
     */
    public Map<String, String> getCopy() {
        return new HashMap<String, String>(m_parsedValues);
    }
    
    /**
     * Resets this value collection, by clearing all data that has been parsed
     * up to this point.
     * 
     */
    public void reset() {
        m_parsedValues.clear();
    }
}
